<?php

/**
 * @file
 * Contains \Drupal\pc\ConnectorFactory.
 */

namespace Drupal\pc;

use Drupal\Core\Session\AccountInterface;
use PhpConsole\Connector;
use PhpConsole\Storage;
use PhpConsole\Handler;
use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\ClassLoader\ApcClassLoader;

class ConnectorFactory {

  /**
   * @var \PhpConsole\Connector
   */
  protected $connector;

  /**
   * Constructs a connector factory object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory object to use.
   * @param ApcClassLoader $class_loader
   *   The class loader.
   * @param \Drupal\Core\Session\AccountInterface $account
   *   The user session.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ApcClassLoader $class_loader, AccountInterface $account) {

    // PhpConsole library does not allow us to set postponed storage after connector is instanced
    // and there is no way to check if it is already set. It might cause an error when the service
    // is being rebuilt. So we have to stick to the same connector instance throughout.
    // @see Connector::setPostponeStorage@see()
    $this->connector = &drupal_static('pc_connector');

    if ($this->connector || !$account->hasPermission('view debug information')) {
      return;
    }

    $settings = $config_factory->get('pc.settings');

    $library_path = $settings->get('library_path');
    $class_loader->add('PhpConsole', DRUPAL_ROOT . '/' . $library_path . '/src');
    // PhpConsole library is not installed.
    if (!class_exists('PhpConsole\Connector')) {
      return;
    }

    $pc_data_file = $config_factory
        ->get('system.file')
        ->get('path.temporary') . '/pc.data';

    Connector::setPostponeStorage(new Storage\File($pc_data_file, FALSE));


    $this->connector = Connector::getInstance();

    if ($settings->get('password_enabled')) {
      $password = $settings->get('password');
      $this->connector->setPassword($password ?: user_password());
    }

    $ips = explode("\n", $settings->get('ips'));
    $ips = array_map('trim', $ips);
    $ips = array_filter($ips, 'strlen');
    $ips && $this->connector->setAllowedIpMasks($ips);

    // Configure eval provider.
    if ($settings->get('password_enabled') && $settings->get('remote_php_execution') && $account->hasPermission('execute remote php code')) {
      $eval_provider = $this->connector->getEvalDispatcher()->getEvalProvider();
      $eval_provider->setOpenBaseDirs([DRUPAL_ROOT]);
      $this->connector->startEvalRequestsListener();
    }

    // Enable error handler.
    if ($settings->get('track_errors')) {
      $handler = Handler::getInstance();
      if (!$handler->isStarted()) {
        $handler->start();
        $this->connector->setSourcesBasePath(DRUPAL_ROOT);
      }
    }

  }

  /**
   * Returns PHP Console connector.
   */
  public function get() {
    return $this->connector;
  }

}
